@@ -14,7 +14,7 @@ from torch import nn, Tensor, device
 from torch.optim import Optimizer
 from torch.utils.data import DataLoader
 import torch.multiprocessing as mp
-from tqdm.autonotebook import tqdm, trange
+from tqdm.autonotebook import trange
 import math
 import queue
 
@@ -35,6 +35,8 @@ class SentenceTransformer(nn.Sequential):
     :param device: Device (like 'cuda' / 'cpu') that should be used for computation. If None, checks if a GPU can be used.
     """
     def __init__(self, model_name_or_path: str = None, modules: Iterable[nn.Module] = None, device: str = None):
+        save_model_to = None
+
         if model_name_or_path is not None and model_name_or_path != "":
             logger.info("Load pretrained SentenceTransformer: {}".format(model_name_or_path))
             model_path = model_name_or_path
@@ -46,11 +48,11 @@ class SentenceTransformer(nn.Sequential):
                     raise AttributeError("Path {} not found".format(model_path))
 
                 model_path = __DOWNLOAD_SERVER__ + model_path + '.zip'
-                logger.info("Try to download model from server: {}".format(model_path))
+                logger.info("Search model on server: {}".format(model_path))
 
             if model_path.startswith('http://') or model_path.startswith('https://'):
                 model_url = model_path
-                folder_name = model_url.replace("https://", "").replace("http://", "").replace("/", "_")[:250].rstrip('.zip')
+                folder_name = model_url.replace("https://", "").replace("http://", "").replace("/", "_")[:250][0:-4] #remove .zip file end
 
                 cache_folder = os.getenv('SENTENCE_TRANSFORMERS_HOME')
                 if cache_folder is None:
@@ -65,8 +67,10 @@ class SentenceTransformer(nn.Sequential):
                 model_path = os.path.join(cache_folder, folder_name)
 
                 if not os.path.exists(model_path) or not os.listdir(model_path):
-                    if model_url[-1] == "/":
-                        model_url = model_url[:-1]
+                    if os.path.exists(model_path):
+                        os.remove(model_path)
+
+                    model_url = model_url.rstrip("/")
                     logger.info("Downloading sentence transformer model from {} and saving it at {}".format(model_url, model_path))
 
                     model_path_tmp = model_path.rstrip("/").rstrip("\\")+"_part"
@@ -79,15 +83,18 @@ class SentenceTransformer(nn.Sequential):
                         os.rename(model_path_tmp, model_path)
                     except requests.exceptions.HTTPError as e:
                         shutil.rmtree(model_path_tmp)
+                        if e.response.status_code == 429:
+                            raise Exception("Too many requests were detected from this IP for the model {}. Please contact info@nils-reimers.de for more information.".format(model_name_or_path))
+
                         if e.response.status_code == 404:
                             logger.warning('SentenceTransformer-Model {} not found. Try to create it from scratch'.format(model_url))
                             logger.warning('Try to create Transformer Model {} with mean pooling'.format(model_name_or_path))
 
+                            save_model_to = model_path
                             model_path = None
                             transformer_model = Transformer(model_name_or_path)
                             pooling_model = Pooling(transformer_model.get_word_embedding_dimension())
                             modules = [transformer_model, pooling_model]
-
                         else:
                             raise e
                     except Exception as e:
@@ -99,12 +106,6 @@ class SentenceTransformer(nn.Sequential):
             if model_path is not None:
                 logger.info("Load SentenceTransformer from folder: {}".format(model_path))
 
-                if os.path.exists(os.path.join(model_path, 'config.json')):
-                    with open(os.path.join(model_path, 'config.json')) as fIn:
-                        config = json.load(fIn)
-                        if config['__version__'] > __version__:
-                            logger.warning("You try to use a model that was created with version {}, however, your version is {}. This might cause unexpected behavior or errors. In that case, try to update to the latest version.\n\n\n".format(config['__version__'], __version__))
-
                 with open(os.path.join(model_path, 'modules.json')) as fIn:
                     contained_modules = json.load(fIn)
 
@@ -125,6 +126,10 @@ class SentenceTransformer(nn.Sequential):
 
         self._target_device = torch.device(device)
 
+        #We created a new model from scratch based on a Transformer model. Save the SBERT model in the cache folder
+        if save_model_to is not None:
+            self.save(save_model_to)
+
 
     def encode(self, sentences: Union[str, List[str], List[int]],
                batch_size: int = 32,
@@ -132,9 +137,8 @@ class SentenceTransformer(nn.Sequential):
                output_value: str = 'sentence_embedding',
                convert_to_numpy: bool = True,
                convert_to_tensor: bool = False,
-               is_pretokenized: bool = False,
                device: str = None,
-               num_workers: int = 0) -> Union[List[Tensor], ndarray, Tensor]:
+               normalize_embeddings: bool = False) -> Union[List[Tensor], ndarray, Tensor]:
         """
         Computes sentence embeddings
 
@@ -144,9 +148,9 @@ class SentenceTransformer(nn.Sequential):
         :param output_value:  Default sentence_embedding, to get sentence embeddings. Can be set to token_embeddings to get wordpiece token embeddings.
         :param convert_to_numpy: If true, the output is a list of numpy vectors. Else, it is a list of pytorch tensors.
         :param convert_to_tensor: If true, you get one large tensor as return. Overwrites any setting from convert_to_numpy
-        :param is_pretokenized: DEPRECATED - No longer used
         :param device: Which torch.device to use for the computation
-        :param num_workers: DEPRECATED - No longer used
+        :param normalize_embeddings: If set to true, returned vectors will have length 1. In that case, the faster dot-product (util.dot_score) instead of cosine similarity can be used.
+
         :return:
            By default, a list of tensors is returned. If convert_to_tensor, a stacked tensor is returned. If convert_to_numpy, a numpy matrix is returned.
         """
@@ -154,8 +158,15 @@ class SentenceTransformer(nn.Sequential):
         if show_progress_bar is None:
             show_progress_bar = (logger.getEffectiveLevel()==logging.INFO or logger.getEffectiveLevel()==logging.DEBUG)
 
+        if convert_to_tensor:
+            convert_to_numpy = False
+
+        if output_value == 'token_embeddings':
+            convert_to_tensor = False
+            convert_to_numpy = False
+
         input_was_string = False
-        if isinstance(sentences, str): #Cast an individual sentence to a list with length 1
+        if isinstance(sentences, str) or not hasattr(sentences, '__len__'): #Cast an individual sentence to a list with length 1
             sentences = [sentences]
             input_was_string = True
 
@@ -165,34 +176,34 @@ class SentenceTransformer(nn.Sequential):
         self.to(device)
 
         all_embeddings = []
-        length_sorted_idx = np.argsort([self._text_length(sen) for sen in sentences])
+        length_sorted_idx = np.argsort([-self._text_length(sen) for sen in sentences])
         sentences_sorted = [sentences[idx] for idx in length_sorted_idx]
 
-        iterator = range(0, len(sentences), batch_size)
-        if show_progress_bar:
-            iterator = tqdm(iterator, desc="Batches")
-
-        for start_index in iterator:
+        for start_index in trange(0, len(sentences), batch_size, desc="Batches", disable=not show_progress_bar):
             sentences_batch = sentences_sorted[start_index:start_index+batch_size]
             features = self.tokenize(sentences_batch)
             features = batch_to_device(features, device)
 
             with torch.no_grad():
                 out_features = self.forward(features)
-                embeddings = out_features[output_value]
 
                 if output_value == 'token_embeddings':
-                    #Set token embeddings to 0 for padding tokens
-                    input_mask = out_features['attention_mask']
-                    input_mask_expanded = input_mask.unsqueeze(-1).expand(embeddings.size()).float()
-                    embeddings = embeddings * input_mask_expanded
+                    embeddings = []
+                    for token_emb, attention in zip(out_features[output_value], out_features['attention_mask']):
+                        last_mask_id = len(attention)-1
+                        while last_mask_id > 0 and attention[last_mask_id].item() == 0:
+                            last_mask_id -= 1
 
-                embeddings = embeddings.detach()
+                        embeddings.append(token_emb[0:last_mask_id+1])
+                else:   #Sentence embeddings
+                    embeddings = out_features[output_value]
+                    embeddings = embeddings.detach()
+                    if normalize_embeddings:
+                        embeddings = torch.nn.functional.normalize(embeddings, p=2, dim=1)
 
-                # fixes for #522 and #487
-                # to avoid oom problems on gpu with large datasets
-                if convert_to_numpy:
-                    embeddings = embeddings.cpu()
+                    # fixes for #522 and #487 to avoid oom problems on gpu with large datasets
+                    if convert_to_numpy:
+                        embeddings = embeddings.cpu()
 
                 all_embeddings.extend(embeddings)
 
@@ -318,11 +329,11 @@ class SentenceTransformer(nn.Sequential):
 
         return None
 
-    def tokenize(self, text: str):
+    def tokenize(self, texts: Union[List[str], List[Dict], List[Tuple[str, str]]]):
         """
-        Tokenizes the text
+        Tokenizes the texts
         """
-        return self._first_module().tokenize(text)
+        return self._first_module().tokenize(texts)
 
     def get_sentence_features(self, *features):
         return self._first_module().get_sentence_features(*features)
@@ -404,12 +415,15 @@ class SentenceTransformer(nn.Sequential):
         a list of ints (which means a single text as input), or a tuple of list of ints
         (representing several text inputs to the model).
         """
-        if isinstance(text, dict):
+
+        if isinstance(text, dict):              #{key: value} case
             return len(next(iter(text.values())))
-        elif len(text) == 0 or isinstance(text[0], int):
+        elif not hasattr(text, '__len__'):      #Object has no len() method
+            return 1
+        elif len(text) == 0 or isinstance(text[0], int):    #Empty string or list of ints
             return len(text)
         else:
-            return sum([len(t) for t in text])
+            return sum([len(t) for t in text])      #Sum of length of individual strings
 
     def fit(self,
             train_objectives: Iterable[Tuple[DataLoader, nn.Module]],
@@ -419,7 +433,7 @@ class SentenceTransformer(nn.Sequential):
             scheduler: str = 'WarmupLinear',
             warmup_steps: int = 10000,
             optimizer_class: Type[Optimizer] = transformers.AdamW,
-            optimizer_params : Dict[str, object]= {'lr': 2e-5, 'eps': 1e-6, 'correct_bias': False},
+            optimizer_params : Dict[str, object]= {'lr': 2e-5},
             weight_decay: float = 0.01,
             evaluation_steps: int = 0,
             output_path: str = None,
@@ -427,7 +441,10 @@ class SentenceTransformer(nn.Sequential):
             max_grad_norm: float = 1,
             use_amp: bool = False,
             callback: Callable[[float, int, int], None] = None,
-            output_path_ignore_not_empty: bool = False
+            show_progress_bar: bool = True,
+            checkpoint_path: str = None,
+            checkpoint_save_steps: int = 500,
+            checkpoint_save_total_limit: int = 1
             ):
         """
         Train the model with the given training objective
@@ -452,7 +469,10 @@ class SentenceTransformer(nn.Sequential):
         :param callback: Callback function that is invoked after each evaluation.
                 It must accept the following three parameters in this order:
                 `score`, `epoch`, `steps`
-        :param output_path_ignore_not_empty: deprecated, no longer used
+        :param show_progress_bar: If True, output a tqdm progress bar
+        :param checkpoint_path: Folder to save checkpoints during training
+        :param checkpoint_save_steps: Will save a checkpoint after so many steps
+        :param checkpoint_save_total_limit: Total number of checkpoints to store
         """
 
         if use_amp:
@@ -506,14 +526,14 @@ class SentenceTransformer(nn.Sequential):
         num_train_objectives = len(train_objectives)
 
         skip_scheduler = False
-        for epoch in trange(epochs, desc="Epoch"):
+        for epoch in trange(epochs, desc="Epoch", disable=not show_progress_bar):
             training_steps = 0
 
             for loss_model in loss_models:
                 loss_model.zero_grad()
                 loss_model.train()
 
-            for _ in trange(steps_per_epoch, desc="Iteration", smoothing=0.05):
+            for _ in trange(steps_per_epoch, desc="Iteration", smoothing=0.05, disable=not show_progress_bar):
                 for train_idx in range(num_train_objectives):
                     loss_model = loss_models[train_idx]
                     optimizer = optimizers[train_idx]
@@ -531,20 +551,6 @@ class SentenceTransformer(nn.Sequential):
                     features, labels = data
 
 
-                    """
-                    num_texts = len(data[0].texts)
-                    texts = [[] for _ in range(num_texts)]
-                    labels = []
-                    for example in data:
-                        labels.append(data.label)
-                        for idx in range(num_texts):
-                            texts[idx].append(example.texts[idx])
-
-                    features = self.tokenize(texts)
-                    features = batch_to_device(features, self._target_device)
-                    labels  = torch.stack(labels).to(self._target_device)
-                    """
-
                     if use_amp:
                         with autocast():
                             loss_value = loss_model(features, labels)
@@ -578,8 +584,20 @@ class SentenceTransformer(nn.Sequential):
                         loss_model.zero_grad()
                         loss_model.train()
 
+                if checkpoint_path is not None and checkpoint_save_steps is not None and checkpoint_save_steps > 0 and global_step % checkpoint_save_steps == 0:
+                    self._save_checkpoint(checkpoint_path, checkpoint_save_total_limit, global_step)
+
+
             self._eval_during_training(evaluator, output_path, save_best_model, epoch, -1, callback)
 
+        if evaluator is None and output_path is not None:   #No evaluator, but output path: save final model version
+            self.save(output_path)
+
+        if checkpoint_path is not None:
+            self._save_checkpoint(checkpoint_path, checkpoint_save_total_limit, global_step)
+
+
+
     def evaluate(self, evaluator: SentenceEvaluator, output_path: str = None):
         """
         Evaluate the model
@@ -604,6 +622,20 @@ class SentenceTransformer(nn.Sequential):
                 if save_best_model:
                     self.save(output_path)
 
+    def _save_checkpoint(self, checkpoint_path, checkpoint_save_total_limit, step):
+        # Store new checkpoint
+        self.save(os.path.join(checkpoint_path, str(step)))
+
+        # Delete old checkpoints
+        if checkpoint_save_total_limit is not None and checkpoint_save_total_limit > 0:
+            old_checkpoints = []
+            for subdir in os.listdir(checkpoint_path):
+                if subdir.isdigit():
+                    old_checkpoints.append({'step': int(subdir), 'path': os.path.join(checkpoint_path, subdir)})
+
+            if len(old_checkpoints) > checkpoint_save_total_limit:
+                old_checkpoints = sorted(old_checkpoints, key=lambda x: x['step'])
+                shutil.rmtree(old_checkpoints[0]['path'])
 
     @staticmethod
     def _get_scheduler(optimizer, scheduler: str, warmup_steps: int, t_total: int):
